#
# dbconsole.py
#
# A "console" client application for dbccp4i
#
# Allow the user to issue commands to the database via
# the dbClientAPI.
#
# Usage:
# > python dbconsole.py
#
#Cvs_Id $Id: dbconsole.py,v 1.2 2007/01/19 13:51:10 wy45 Exp $
#--------------------------------------------------------------

import ccp4i
import dbClientAPI
import exceptions
import sys
import os

def PrintResult(result):
    print 'Result:'
    if type(result) == list:
        for item in result:
            print item
    else:
        print result

# Acquire the user id
user = ccp4i.GetUserId()
if user == "":
    print "No user name: setting to 'unknown'"
    user = "unknown"
else:
    print "User is '"+user+"'"

# Try to connect to the handler
try:
    dbClientAPI.DbStartHandler()
    conn = dbClientAPI.handlerconnection()
    conn.DbRegister(user,'dummy',True)
except exceptions.Exception,e:
    print "dbconsole: connection failed with exception:"
    print str(e)
    sys.exit()

# Set up a broadcast reporter
conn.BroadcastReporter()
 
# Acquire the location of the Client API source file
# Use the BIOXHIT_TOP environment variable
Bioxhit_top = os.environ.get("BIOXHIT_TOP")
if Bioxhit_top == "":
    print "BIOXHIT_TOP is not set"
    sys.exit()
dbClientAPI_file = os.path.join(Bioxhit_top,"dbccp4i","ClientAPI","dbClientAPI.py")
f = open(dbClientAPI_file,"r")
print 'The functions in handlerconnection class are:\n'
while 1:
    line = f.readline()
    if not line: break
    
    if 'def ' in line:
        if '__init__' in line:
            continue
        elif 'request_wrapper' in line:
            continue
        elif 'parse_response' in line:
            continue
        elif 'GetEnvVar' in line:
            continue
        elif 'DbStartHandler' in line:
            continue
        else:
            print line
f.close()

print 'Usage: command arg1 arg2...'
print 'For example: OpenDatabase MYPROJECT'
while 1:
    print '\nEnter command & arguments: (Type \'exit\' to stop) '
    input = raw_input('% ')
    if 'exit' in input:
        break
    else:
        try:
            input = input.split(' ')
            if input[0] == 'ListProjects':
                result = conn.ListProjects()
                PrintResult(result)
                
            elif input[0] == 'CheckServerStatus':
                result = conn.CheckServerStatus()
                PrintResult(result)
                
            elif input[0] == 'OpenDatabase':
                if len(input) != 2:
                    print 'Wrong argument'
                    print 'Usage: OpenDatabase project'
                    continue
                result = conn.OpenDatabase(input[1])
                PrintResult(result)
                
            elif input[0] == 'HandlerDisconnect':
                result = conn.HandlerDisconnect()
                PrintResult(result)
                
            elif input[0] == 'ShutDown':
                result = conn.ShutDown()
                PrintResult(result)
                
            elif input[0] == 'CloseDatabase':
                if len(input) != 2:
                    print 'Wrong argument'
                    print 'Usage: CloseDatabase project'
                    continue
                result = conn.CloseDatabase(input[1])
                PrintResult(result)
                
            elif input[0] == 'NewRecord':
                if len(input) != 2:
                    print 'Wrong argument'
                    print 'Usage: NewRecord project'
                    continue
                result = conn.NewRecord(input[1])
                PrintResult(result)
                
            elif input[0] == 'DeleteRecord':
                if len(input) != 3:
                    print 'Wrong argument'
                    print 'Usage: DeleteRecord project jobid'
                    continue
                result = conn.DeleteRecord(input[1],input[2])
                PrintResult(result)
                
            elif input[0] == 'ItemExists':
                if len(input) != 4:
                    print 'Wrong argument'
                    print 'Usage: ItemExists project jobid item'
                    continue 
                result = conn.ItemExists(input[1],input[2],input[3])
                PrintResult(result)
                
            elif input[0] == 'SetData':
                if len(input) != 5:
                    print 'Wrong argument'
                    print 'Usage: SetData project jobid item value'
                    continue 
                result = conn.SetData(input[1],input[2],input[3],input[4])
                PrintResult(result)
                
            elif input[0] == 'GetData':
                if len(input) != 4:
                    print 'Wrong argument'
                    print 'Usage: GetData project jobid item'
                    continue 
                result = conn.GetData(input[1],input[2],input[3])
                PrintResult(result)
                
            elif input[0] == 'GetDescription':
                if len(input) != 5:
                    print 'Wrong argument'
                    print 'Usage: GetDescription project job_list db_display db_display_format'
                    continue 
                result = conn.GetDescription(input[1],input[2],input[3],input[4])
                PrintResult(result)
                
            elif input[0] == 'SelectJobs':
                if len(input) != 4:
                    print 'Wrong argument'
                    print 'Usage: SelectJobs project item pattern'
                    continue 
                result = conn.SelectJobs(input[1],input[2],input[3])
                PrintResult(result)
                
            elif input[0] == 'DeleteProject':
                if len(input) != 2:
                    print 'Wrong argument'
                    print 'Usage: DeleteProject project'
                    continue 
                result = conn.DeleteProject(input[1])
                PrintResult(result)
                
            elif input[0] == 'GetProjectDir':
                result = conn.GetProjectDir()
                PrintResult(result)
                
            elif input[0] == 'ListDefDirs':
                result = conn.ListDefDirs()
                PrintResult(result)
                
            elif input[0] == 'GetDefDir':
                if len(input) != 2:
                    print 'Wrong argument'
                    print 'Usage: GetDefDir def_dir'
                    continue 
                result = conn.GetDefDir(input[1])
                PrintResult(result)
                  
            elif input[0] == 'GetNJOB':
                if len(input) != 2:
                    print 'Wrong argument'
                    print 'Usage: GetNJOB project'
                    continue  
                result = conn.GetNJOB(input[1])
                PrintResult(result)
                
            elif input[0] == 'GetNProjects':
                result = conn.GetNProjects()
                PrintResult(result)

            elif input[0] == 'NewProjectsql':
                result = conn.NewProjectsql(input[1])
                PrintResult(result)

            elif input[0] == 'OpenDBsql':
                result = conn.OpenDBsql(input[1],input[2])
                PrintResult(result)

            elif input[0] == 'NewDBsql':
                result = conn.NewDBsql(input[1],input[2])
                PrintResult(result)
                
            elif input[0] == 'NewJobsql':
                result = conn.NewJobsql(input[1])
                PrintResult(result)

            elif input[0] == 'SetDatasql':
                result = conn.SetDatasql(input[1],input[2],input[3])
                PrintResult(result)

            elif input[0] == 'GetDatasql':
                result = conn.GetDatasql(input[1],input[2])
                PrintResult(result)

            elif input[0] == 'CloseDBsql':
                result = conn.CloseDBsql()
                PrintResult(result)

            elif input[0] == 'SetProjectOwner':
                result = conn.SetProjectOwner(input[1],input[2])
                PrintResult(result)

            elif input[0] == 'SetProjectPath':
                result = conn.SetProjectPath(input[1],input[2])
                PrintResult(result)
            
                PrintResult(result)
                
            else:
                print 'Wrong command, please try again.'
                continue
        except exceptions.Exception,e:
            print str(e)
#            conn.HandlerDisconnect()
        
conn.HandlerDisconnect()

