#
#     Copyright (C) 2006
#
#     This code is distributed under the terms and conditions of the
#     CCP4 licence agreement as `Part 1' (Annex 2) software.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#====================================================================
# SQLite Interface - dbapi_sqlite.py
#
# Class for interacting SQLite database 
#
# Wanjuan Yang March 06
#
#===================================================================
#CCP4i_cvs_Id $Id: dbapi_sqlite.py,v 1.13 2007/01/18 11:51:31 wy45 Exp $


__version__ = "$Revision: 1.13 $"


#######################################################################
# Import modules that this module depends on
#######################################################################

from pysqlite2 import dbapi2 as sqlite
import exceptions
import sys
import os

BIOXHIT_TOP = os.environ.get("BIOXHIT_TOP")
if BIOXHIT_TOP == None:
    print "BIOXHIT_TOP not set"

class DB:
    
    def __init__(self):
        self.open = False
        self.conn = None
        self.cur = None
        self._db = ""

    def OpenDB(self,dbname,dir):
        self._db = dir + "/" + dbname
        
        if os.path.isfile(self._db):                
            try:
                self.conn = sqlite.connect(self._db)
                self.cur = self.conn.cursor()
                self.open = True
                result = True
            
            except exceptions.Exception,e:
                print '# DB message: catch exception: ', str(e)
                result = 'DB exception:'+ str(e)
        
        else:
            print '# DB message: DB does not exists'
            result =' DB does not exist'

        return result

    def NewDB(self,dbname,dir):
        self._db = dir + "/" + dbname

        try:
            self.conn = sqlite.connect(self._db)
           
            self.cur = self.conn.cursor()

            schemafile = BIOXHIT_TOP + '/dbccp4i/dbccp4i/schema.sql'
            sqllist = getsql(schemafile)
            for sql in sqllist:
                print sql
                self.cur.execute(sql)
            self.open = True
            result = True
            
        except exceptions.Exception,e:
            result = 'DB exception: '+ str(e)

        return result
    
    def CloseDB(self):
        if self.open:
            try:
                self.conn.close()
                result = True
            except exceptions.Exception,e:
                result = 'DB exception: '+ str(e)
        else:
            result = 'DB not open'

        return result

    def IsOpen(self):
        return self.open

    def new_record(self,table):
        sql = 'insert into '+table+'('+table+'Id) values (Null)'
        
        try:
            self.cur.execute(sql)
            self.conn.commit()
            sql2 = 'select '+table+'Id from '+table
            self.cur.execute(sql2)
            self.conn.commit()
            pre_result = self.cur.fetchall()
            result = str(pre_result[-1][0])
                  
        except exceptions.Exception,e:
            print '# DB message: catch exception: ', str(e)
            result ='DB exception: '+str(e)

        return result

    def set_table_attribute(self,table,id,attribute,value):
        try:
            if self.open:
                sql = 'update '+table+' set '+attribute + '=\''+ value+'\' where '+table+'Id='+ str(id)
                print sql
                self.cur.execute(sql)
                self.conn.commit()
                result = True
            else:
                result = 'DB not open yet'
                
        except exceptions.Exception,e:
            print '# DB message: catch exception: ', str(e)
            result =  'error: '+ str(e)
        return result
        
        
    def get_table_attribute(self,table,id,attribute):
        try:
            if self.open:
                sql = 'select '+attribute + ' from '+table+' where '+table+'Id= '+str(id)
                print sql
                self.cur.execute(sql)
                self.conn.commit()
                resultset = self.cur.fetchall()
                result = str(resultset[-1][0])
                
            else:
                result = 'DB not open yet'
                
        except exceptions.Exception,e:
            print '# DB message: catch exception when get data: ', str(e)
            result =  'error:'+str(e)
            
        return result

    def get_table_primary_key(self,table,condition):
        try:
            if self.open:
                sql = 'select '+table+'Id'+' from '+table+' '+condition 
                self.cur.execute(sql)
                self.conn.commit()
                resultset = self.cur.fetchall()
                if resultset != []:
                    result = str(resultset[-1][0])
                else:
                    result = resultset
                
            else:
                result = 'DB not open yet'
                
        except exceptions.Exception,e:
            print '# DB message: catch exception: ', str(e)
            result =  'error:'+str(e)
            
        return result
  
    def GetProjectId(self,projectname):
        try:
            if self.open:
                sql = 'select projectId from Project where ProjectName = \''+projectname+'\''
                print sql
                self.cur.execute(sql)
                self.conn.commit()
                resultset = self.cur.fetchall()
                if resultset != []:
                    result = str(resultset[-1][0])
                else:
                    result = resultset
                
            else:
                result = 'DB not open yet'
                
        except exceptions.Exception,e:
            print '# DB message: catch exception: ', str(e)
            result =  'error:'+str(e)
            
        return result
            

    def GetProjectname(self,jobid):
        try:
            if self.open:
                sql1 = 'select ProjectId from Job where JobId = '+str(jobid)
                self.cur.execute(sql1)
                resultset = self.cur.fetchall()
                projectid = resultset[0][0]
                sql2 = 'select ProjectName from Project where ProjectId = '+ str(projectid)
                self.cur.execute(sql2)
                resultset = self.cur.fetchall()
                projectname = resultset[0][0]
                result = projectname
            else:
                result = 'DB not open'
                
        except exceptions.Exception,e:
            print '# DB message: catch exception when get projectname: ', str(e)
            result =  'error:'+str(e)
            
        return result

    def GetProjectList(self):
        try:
            if self.open:
                sql = 'select ProjectName from Project'
                self.cur.execute(sql)
                resultset = self.cur.fetchall()
                result = []
                for item in resultset:
                    result.append(str(item)[3:-3])              
            else:
                result = 'DB not open'
                
        except exceptions.Exception,e:
            print '# DB message: catch exception: ', str(e)
            result =  'error:'+str(e)
            
        return result   

    def GetJobList(self,projectid):
        try:
            if self.open:
                sql = 'select * from job where ProjectId = '+projectid
                self.cur.execute(sql)
                resultset = self.cur.fetchall()
                result = []
                for item in resultset:
                    print item
                    if 'u\'' in item:
                        result.append(str(item)[3:-3])
                    else:
                        result.append(item)
            else:
                result = 'DB not open'
                
        except exceptions.Exception,e:
            print '# DB message: catch exception: ', str(e)
            result =  'error:'+str(e)
            
        return result   

    def GetRowofTable(self,table,id):
        try:
            if self.open:
                sql = 'select * from '+table+' where '+table+'Id ='+id
                self.cur.execute(sql)
                resultset = self.cur.fetchall()
                result = []
                print resultset
                for item in resultset:               
                    if 'u\'' in item:
                        result.append(str(item)[3:-3])
                    else:
                        result.append(item)
            else:
                result = 'DB not open'
                
        except exceptions.Exception,e:
            print '# DB message: catch exception: ', str(e)
            result =  'error:'+str(e)
            
        return resultset   

def getsql(filename):
    fin = open(filename)
    sql = ''
    while 1:
        line = fin.readline()
        if not line: break

        sql = sql+line

    list = sql.split(';')
    return list

if __name__ == '__main__':
    db = DB()
    db.NewDB('testdb','/home/wy45/work/testarea/DB')
