#
#     Copyright (C) 2006 
#
#     This code is distributed under the terms and conditions of the
#     CCP4 licence agreement as `Part 1' (Annex 2) software.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#====================================================================
# Driver for the database handler
#
# Wanjuan Yang March 06
#
#====================================================================

#CCP4i_cvs_Id $Id: dbccp4i.py,v 1.42 2007/02/14 17:02:36 wy45 Exp $

######################################################################

__version__ = "$Revision: 1.42 $"


#######################################################################
# Import modules that this module depends on
#######################################################################

import socket
import sys
import datetime
import thread
from threading import *
import threading
import exceptions
import os

try:
    
    import xml.sax.saxutils as su
    import xml.dom.minidom
    
except exceptions.Exception,x:

    print "One or more Python modules failed to load, dbccp4i cannot start."
    print "Pleae check your Python installation, or report this problem to\n"+\
          "the developers."
    sys.exit()

#load the modules of dealing with def files. Check CCP4_TOP & BIOXHIT_TOP environment variables

CCP4i_top = os.environ.get("CCP4I_TOP")
if CCP4i_top == None:
    print "Please set CCP4I_TOP enviornment variable first"
    sys.exit()

import DBcommand

try:
    import ccp4i
    import manager
    from manager import *
    def_flag = True
    
except exceptions.Exception,x:
    def_flag = False

# load API for SQLite
try:
    import dbapi_sqlite 
    sql_flag = True
    
except exceptions.Exception,x:
    sql_flag = False


class MyThread(threading.Thread):
    
    def __init__(self,sock,addr):
        threading.Thread.__init__(self)
        self._sock = sock
        self._addr = addr

        ### valid command list
        # Admin commands
        self._adminCommand = [ 'ShutDown','DbDisconnect','DbRegister','DbRequestStatus','DbSupported' ]

        # Commands for def backend
        self._defCommand = [ 'DbOpen','NewDb','DbNewRecord', 'DbDeleteRecord','DbSetData','Updatetime','DbGetData','DbItemExists','DbSelectRecords','ListJobs','DbReturnRecord','DbClose','ListProjects','DeleteProject','GetProjectDir','ListDefDirs','GetDefDir','GetNJOB','GetNProjects','GetNextJobList','GetAllLinks','GetLinks','DbGetListofRecords','ImportProject','GetAllChildren','GetAllParents','GetParents','GetChildren']

        # Commands for sqlite backend
        self._sql_newrecord_command = ['NewRecord','NewProject_sql','NewJob_sql']
        self._sql_setdata_command = [ 'SetJobData_sql','SetData']
        self._sql_command = ['NewDB_sql','OpenDB_sql','GetJobData_sql', 'CloseDB_sql', 'GetProjectId','GetData','GetProjectList','GetJobList','GetRowofTable']
        self._validCommand = self._adminCommand + self._defCommand + self._sql_newrecord_command+ self._sql_setdata_command +self._sql_command
        
      #  socket.setdefaulttimeout(300)
        self._running = True

        ## database and user info
        self._sqldb = None          #used to store sql database object
        
        ## client info
        self.c = manager.Client(sock,addr)

        ## lock
        self.lock = thread.allocate_lock()

        
    def set_flag(self):
        self._running = False

    def run(self):
        # set initial data
        data = ""
        while not en.isSet():
           try:
                
             data = data + self._sock.recv(1024)

             if data == '':
                 # empty string, remove the client
                 ProjectManager.DelClient(self.c)
                 print 'Handler received empty string: lost client '+str(self._addr)+'?'
                 print "Removing client from handler"
                 break

             # check if this is the whole request
             elif "\n" not in data:
                 continue
                 
             else: # unwrap the data
               messager.output('# Handler message: in connection '+str(self._addr) )
               messager.output(' receive request: ')
               messager.output(data)
               try:
                    doc = xml.dom.minidom.parseString(data)
                    com = doc.getElementsByTagName("command")
                    command = com[0].childNodes[0].data
                    argu = doc.getElementsByTagName("argument")
               
                    rid = doc.getElementsByTagName("request_id")            
                    if rid != []:
                        id = rid[0].childNodes[0].data
                    # reset data
                    data = ""
                    
               except exceptions.Exception,e:
                    messager.output('# Handler message: in connection '+str(self._addr))
                    messager.output(' catch exception when parse XML, not well form request:')
                    messager.output(str(e))
                    status = 'ERROR'
                    result = 'not well form request'
                    response = response_wrapper(status,result,'#')
                    #try:
                    self._sock.sendall(response)
                    messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                    #except exceptions.Exception,e:
                    #print '#: Handler message: response could not send ',str(e)
                    continue
                 
                    
               # test whether is a valid commands
               if command not in self._validCommand:
                   status = 'ERROR'
                   result = 'not valid command'
                   response = response_wrapper(status,result,id)
                   
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                   except exceptions.Exception,x:
                       messager.output('# Handler message: in connection'+ str(self._addr))
                       messager.output('message couldnot sent')
                       
               # process difference commands
               elif command == "ShutDown":
                   # receive shutdown message, tell main, set flag 
                   # and exit itself directly
        
                   ProjectManager.DelClient(self.c)
                   en.set()    
                   break

               elif command == "DbRequestStatus":
                   result = 'Active'
                   status = 'OK'
                   response = response_wrapper(status,result,id)
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler messager: in connection ' + str(self._addr) + ' sent response: ' + response)
                   except exceptions.Exception,x:
                       messager.output ('#: error when RequestStatus:')
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)
                       break
               
               elif command == "DbDisconnect":
                   # receive close message, close all projects it opened, del from list, set flag self._b, and exit directly
                 
                   result = True
                   status = 'OK'
                   
                   response = response_wrapper(status,result,id)

                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)

                   except exceptions.Exception,x:
                       messager.output(str(x))
                         
                   ProjectManager.DelClient(self.c)
                   break

               elif command == "DbRegister":
                   # check arguments number
                   if len(argu)<2:
                       result = 'Arguments should be: username pid [broadcast]'
                       status = 'ERROR'
                   else:                       
                       try:
                           username = argu[0].childNodes[0].data
                           pid = argu[1].childNodes[0].data
                               
                           # create a client object
                           self.c = manager.Client(self._sock,self._addr)
                           self.c.SetAuth(True)
                           self.c.AddUser(username)
                           
                           # set the broadcast_flag if specified
                           if len(argu) > 2:
                               broadcast_flag = argu[2].childNodes[0].data
                               if broadcast_flag == 'False':
                                   self.c.SetBroadcast(False)
                       
                           #add to the clientlist
                           ProjectManager.AddClient(self.c)
                     
                           # create a user
                           if ProjectManager.IsUserRegistered(username)!= True:
        
                               ProjectManager.CreateUser(username,pid,self.c)
                               if ProjectManager.directory.islocked(): #force to overwrite if locked
                                   result = ProjectManager.directory.load(force=True)
                               
                                   if result == True:
                                       ProjectManager.dataloaded = True
                                       result = 'data locked, force to override, data loaded'
                                       #set a flag for this type of result, used for find the result
                                       lock = 1
                                   else:
                                       result = 'Data does not exist'
                               else: #load data 
                                   result = ProjectManager.directory.load()
                                   if result == False:
                                       result = 'Data does not exist'
                                   else:
                                       ProjectManager.dataloaded = True
                           else:   
                               ProjectManager.AttachClientToUser(username,self.c)
                               result = True

                           if result == True or lock == 1:
                               status = 'OK'
                           else:
                               status = 'ERROR'
                     
                       except exceptions.Exception,x:
                           result = str(x)
                           status='ERROR'

                   response = response_wrapper(status,result,id)

                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message: in connection ' + str(self._addr) + ' sent  response: ' + response)
                   except exceptions.Exception,x:
                       messager.output('#: error when DbRegister:')
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)
                       break

               elif command == 'DbSupported':
                   try:
                       if len(argu) < 1:
                           result = 'Argument should be: dbtype '
                           status = 'ERROR'
                       else:
                           dbtype = argu[0].childNodes[0].data
                           if dbtype == 'def' or dbtype == 'Def' or dbtype == 'DEF':
                               if def_flag == True:
                                   result = True
                                   status = 'OK'
                               else:
                                   result = False
                                   status = 'ERROR'
                           elif dbtype == 'sqlite' or dbtype == 'SQLite' or dbtype == 'SQLITE':
                               if sql_flag == True:
                                   result = True
                                   status = 'OK'
                               else:
                                   result = False
                                   status = 'ERROR'
                           else:
                               result = False
                               status = 'ERROR'
                   except exceptions.Exception,x:
                       status = 'ERROR'
                       result = str(x)
                   response = response_wrapper(status,result,id)
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                   except exceptions.Exception,x:
                       messager.output('#: error when request DbSupported')
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)
                       break
                       
               elif command == 'ListProjects':
                   try:
                       result = ProjectManager.directory.listprojects()
                       status = 'OK'
                    
                   except exceptions.Exception,x:
                       result = str(x)
                       status = 'ERROR'
                   response = response_wrapper(status,result,id)
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                   except exceptions.Exception,x:
                       messager.output('#: error when request ListProjects')
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)
                       break

               elif command == 'GetNProjects':
                   try:
                       result = ProjectManager.directory.n_projects()
                       status = 'OK'
                    
                   except exceptions.Exception,x:
                       result = str(x)
                       status = 'ERROR'
                   response = response_wrapper(status,result,id)
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                   except exceptions.Exception,x:
                       messager.output('#: error when request GetNProjects')
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)
                       break
               
               elif command == 'DeleteProject':
                   try:
                       if len(argu) < 1:
                           result = 'Arguments should be: projectname'
                           status = 'ERROR'
                       else:
                           projectname = argu[0].childNodes[0].data
                           result = ProjectManager.directory.deleteprojectref(projectname)
                           if result != False:
                               status = 'OK'
                           else:
                               status = 'ERROR'

                   except exceptions.Exception,x:
                       result = str(x)
                       status = 'ERROR'
                       
                   response = response_wrapper(status,result,id)
                   
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                   except exceptions.Exception,x:
                       messager.output('#: error when request DeleteProject')
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)
                       break
                   
               elif command == 'ImportProject':
                   try:
                       if len(argu) < 2:
                           result = 'Arguments should be: alias path'
                           status = 'ERROR'
                       else:
                           alias = argu[0].childNodes[0].data
                           path = argu[1].childNodes[0].data
                           result = ProjectManager.directory.addprojectref(alias,path)
                           if result != False:
                               status = 'OK'
                           else:
                               status = 'ERROR'

                   except exceptions.Exception,x:
                       result = str(x)
                       status = 'ERROR'
                       
                   response = response_wrapper(status,result,id)
                   
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                   except exceptions.Exception,x:
                       messager.output('#: error when request ImportProject')
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)
                       break

               elif command == 'GetProjectDir':
                   try:
                       if len(argu) < 1:
                           result = 'Arguments should be: projectname'
                           status = 'ERROR'
                       else:
                           projectname = argu[0].childNodes[0].data
                           result = ProjectManager.directory.projectdir(projectname)
                           if result != "":
                               status = 'OK'
                           else:
                               status ='ERROR'
                   except exceptions.Exception,x:
                       result = str(x)
                       status ='ERROR'
                   
                   response = response_wrapper(status,result,id)
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response) 
                   except exceptions.Exception,x:
                       messager.output('#: error when request GetProjectDir')
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)
                       break

               elif command == 'ListDefDirs':
                   try:
                       result = ProjectManager.directory.listdefdirs()
                       status = 'OK'
                   except exceptions.Exception,x:
                       result = str(x)
                       status ='ERROR'
                   response = response_wrapper(status,result,id)
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response) 
                   except exceptions.Exception,x:
                       messager.output('#: error when request ListDefDirs')
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)
                       break
               
               elif command == 'GetDefDir':
                   try:
                       if len(argu) < 1:
                           result = 'Arguments should be: def_dir'
                           status = 'ERROR'
                       else:
                           def_dir = argu[0].childNodes[0].data
                           result = ProjectManager.directory.projectdir(def_dir)
                           if result == "":
                               result = ProjectManager.directory.defdir(def_dir)
                       
                           if result != None:
                               status = 'OK'
                           else:
                               status ='ERROR'
                   except exceptions.Exception,x:
                       result = str(x)
                       status ='ERROR'
                   response = response_wrapper(status,result,id)
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                   except exceptions.Exception,x:
                       messager.output('#: error when GetDefDir')
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)
                       break
               
               elif command == 'NewDb':
                   
                   if def_flag == True:
                       if self.c.GetAuth() == True:
                           try:
                               if len(argu) < 2:
                                   result = 'Arguments should be: projectname dir'
                                   status = 'ERROR'
                               else:
                                   projectname = argu[0].childNodes[0].data
                                   dir = argu[1].childNodes[0].data
                                   result = ProjectManager.NewProject(projectname,dir,self.c)
                                   if result == True:
                                       status = 'OK'
                                   else:
                                       status = 'ERROR'                           
                           except exceptions.Exception,e:
                               messager.output('# Handler message: catch exception when create db:')
                               messager.output(str(e))
                               status = 'ERROR'
                               result = 'no such database: '+str(e)
                       else:
                           result = 'Not authorised yet'
                           status = 'ERROR'
                   else:
                      result = 'def not supported'
                      status = 'ERROR'
                      
                   response = response_wrapper(status,result,id)
            
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)                         
                   except exceptions.Exception,x:
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)
                       break    

               ### if open def db, get an def db object
               elif command == 'DbOpen':
                  if def_flag == True:
                         ## check if the client is allowed to do so
                       if self.c.GetAuth() == True:                   
                           try:
                               # get parameter, test how many arguments
                               argu_num = len(argu)

                               if argu_num < 1:
                                   result = 'Arguments should be: projectname [grablock]'
                                   status = 'ERROR'
                               else:
                                   projectname = argu[0].childNodes[0].data
                                   if argu_num >1:
                                       grablock = argu[1].childNodes[0].data
                                       
                                       if grablock == '-grablock' or grablock == "True":
                                           grablock = True
                                       else:
                                           grablock = False
                                   else:
                                       grablock = False
                                       # check whether project opened, if not, add the project to client
                                   if self.c.IsProjectOpened(projectname):
                                       result = 'Project already opened'

                                   else:
           
                                       result = ProjectManager.OpenProject(projectname,self.c,grablock)
                                       # when open db ok, add to the list
                                       if result == True:
                                           self.c.AddProject(projectname)
                                       
                           except exceptions.Exception,e:
                               messager.output('# Handler message: catch exception when open db:')
                               messager.output(str(e))
                               result = 'no such database: '+str(e)
                          
                       else:
                           result = 'Not authorised yet'
                  else:
                      result = 'def not supported'
               
                  if result == True:
                      status = 'OK'
                  else:
                      status = 'ERROR'    
                  response = response_wrapper(status,result,id)
                  
                  try:
                      self._sock.sendall(response)
                      messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)                          
                  except exceptions.Exception,x:
                      messager.output(str(x))
                      ProjectManager.DelClient(self.c)
                      break
               
               elif command == 'DbNewRecord':
                   try:
                       argu_num = len(argu)
                       if argu_num < 1:
                           result = ''
                           status = 'ERROR'
                       else:
                           projectname = argu[0].childNodes[0].data
                           # get a lock
                           self.lock.acquire(True)
                           p = ProjectManager.GetProject(projectname,self.c)
                           if p != 'not register to the project yet' and p != 'DB not open yet': 
                               if argu_num == 1:
                                   result = p.fileobj.newjob()
                               elif argu_num == 2:
                                   taskname = argu[1].childNodes[0].data       
                                   result = p.fileobj.newjob(taskname)
                               elif argu_num == 3:
                                   taskname = argu[1].childNodes[0].data
                                   status = argu[2].childNodes[0].data         
                                   result = p.fileobj.newjob(taskname,status)
                               else:
                                   taskname = argu[1].childNodes[0].data
                                   status = argu[2].childNodes[0].data
                                   title = argu[3].childNodes[0].data
                                   result = p.fileobj.newjob(taskname,status,title)
                               self.lock.release()
                               if result == False or result == -1:
                                   status = 'ERROR'
                               else:
                                   status = 'OK'
                                
                           else:
                               result = p
                               status = 'ERROR'
                   
                   except exceptions.Exception,e:
                       # release lock
                       self.lock.release()
                       messager.output('# Handler message: catch exception when new record:')
                       messager.output(str(e))
                       status = 'ERROR'
                       result = 'Handler exception: '+str(e)

                   response = response_wrapper(status,result,id)
                   
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                       ## if update sucessfully, then send broadcast message
                       if status =='OK':
                           message = '<db_broadcast><message>Update '+ projectname + ' '+ str(result) + '</message></db_broadcast>\n'
                           messager.output('# Handler message : broadcast '+message )
                           # check project associated clients, send broadcast message, when to send?
                           ProjectManager.broadcast(message,projectname)
    
                         
                   except exceptions.Exception,x:
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)
                       break                 
      
               elif command == 'DbDeleteRecord':
                   try:
                       if len(argu) < 2:
                           result = 'Arguments should be: projectname jobid'
                           status = 'ERROR'
                       else:
                           # get parameter
                           projectname =  argu[0].childNodes[0].data
                           jobid = argu[1].childNodes[0].data
                           self.lock.acquire(True)
                           p = ProjectManager.GetProject(projectname,self.c)
                           if p != 'not register to the project yet' and p != 'DB not open yet':
                               result = p.fileobj.deletejob(int(jobid))
                           else:
                               result = p
                           self.lock.release()

                   except exceptions.Exception,x:
                       self.lock.release()
                       messager.output('# Handler message: catch exception when delete data:')
                       messager.output(str(x))
                       result = 'Handler exception: '+str(x)
                       
                   if result == True:
                       status = 'OK'
                   else:
                       status = 'ERROR'
                   
                   response = response_wrapper(status,result,id)
                   
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                       if status == 'OK':
                           message = '<db_broadcast><message>Update '+ projectname + ' ' + jobid + '</message></db_broadcast>\n'
                           messager.output('# Handler message : broadcast '+message )
                           ProjectManager.broadcast(message,projectname)
                         
                   except exceptions.Exception,x:
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)
                       break    
                  
               elif command == 'DbSetData':
                   try:
                       argu_num = len(argu)
                       if argu_num < 4:
                           result = 'Arguments should be: projectname jobid item value [item1 value1 ...]'
                           status = 'ERROR'
                       else:
                           
                           projectname =  argu[0].childNodes[0].data
                      
                           # get jobid  
                           jobid = argu[1].childNodes[0].data
                           self.lock.acquire(True)
                           p = ProjectManager.GetProject(projectname,self.c)
                           if p != 'not register to the project yet' and p != 'DB not open yet': 
                               for i in range((argu_num-1)/2):
                                   item = argu[2*i+2].childNodes[0].data
                                   value = argu[2*i+3].childNodes[0].data      
                                   result = p.fileobj.setdata(int(jobid),item,value)
                           else:
                               result = p
                           self.lock.release()
                       
                   except exceptions.Exception,e:
                       self.lock.release()
                       messager.output('# Handler message: catch exception when set data:')
                       messager.output(str(e))
                       status = 'ERROR'
                       result = 'Handler exception: '+str(e)
                       
                   if result == True:
                       status = 'OK'
                   else:
                       status = 'ERROR'
                       
                   response = response_wrapper(status,result,id)
                   
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                       if status == 'OK':
                           message = '<db_broadcast><message>Update '+ projectname + ' ' + jobid + '</message></db_broadcast>\n'
                           messager.output('# Handler message : broadcast '+message )
                           # check project associated clients, send broadcast message, when to send?
                           ProjectManager.broadcast(message,projectname)
                           
                   except exceptions.Exception,x:
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)
                       break                 
                                          
               elif command == 'DbClose':
                   try:
                       if len(argu) < 1:
                           result = 'Arguments should be: projectname'
                           status = 'ERROR'
                       else:
                           projectname = argu[0].childNodes[0].data
                           # Close the project & delete project
                     
                           if projectname in self.c.GetProjects():
                               self.c.DeleProject(projectname)
                               ProjectManager.CloseProject(projectname,self.c)
                               result = True
                           else:
                               result = 'Project not opened'
                    
                   except exceptions.Exception,e:
                       messager.output('# Handler message: catch exception when close DB:')
                       messager.output(str(e))
                       result = str(e)
                       
                   if result == True:
                       status = 'OK'

                   else:
                       status = 'ERROR'

                   response = response_wrapper(status,result,id)

                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                   except exceptions.Exception,x:
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)   
                       break
                   
               ### if open sqldb, get sqldb object
               elif command == 'OpenDB_sql':
                   if sql_flag == True:
                       try:
                           # get parameter
                           if len(argu) < 2:
                               result = 'Arguments should be: projectname dir'
                               status = 'ERROR'
                           else:
                               projectname = argu[0].childNodes[0].data
                               dir = argu[1].childNodes[0].data
                               dbname = dir +"/"+projectname
                       
                               # call db API
                               self._sqldb = dbapi_sqlite.DB()
                               result = self._sqldb.OpenDB(projectname,dir)
                               if result == True:
                                   # add to client info
                                   self.c.SetDb(dbname)
                                   
                                   status = 'OK'
                               else:
                                   status = 'ERROR'
                               
                       except exceptions.Exception,e:
                           messager.output('# Handler message: catch exception when Open SQL db:')
                           messager.output(str(e))
                           status = 'ERROR'
                           result = 'no such database: '+str(e)
                   else:
                       status = 'ERROR'
                       result = 'no SQLite support'

                   response = response_wrapper(status,result,id)
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                   except exceptions.Exception,x:
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)   
                       break
                   
               elif command == 'NewDB_sql':
                   if sql_flag == True:
                       try:
                           # get parameter
                           if len(argu) < 2 :
                               result = 'Arguments should be: projectname dir'
                               status = 'ERROR'
                           else:
                               projectname = argu[0].childNodes[0].data
                               dir = argu[1].childNodes[0].data
                               dbname = dir +"/"+projectname
                       
                               # call db API
                               self._sqldb = dbapi_sqlite.DB()
                               result = self._sqldb.NewDB(projectname,dir)
                               if result == True:                           
                                   # add to client info
                                   self.c.SetDb(dbname)
                           
                                   status = 'OK'
                               else:
                                   status = 'ERROR'
                       except exceptions.Exception,e:
                           messager.output('# Handler message: catch exception when Open SQL db:')
                           messager.output(str(e))
                           status = 'ERROR'
                           result = 'no such database: '+str(e)
                   else:
                       status = 'ERROR'
                       result = 'no SQLite support'

                   response = response_wrapper(status,result,id)
                   
                   try:
                       self._sock.sendall(response)
                       messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                   except exceptions.Exception,x:
                       messager.output(str(x))
                       ProjectManager.DelClient(self.c)   
                       break
                       
               ### if other commands, deal with either def or sqldb
               else:
                   # deal with the requests
                   if command in self._defCommand:
                       if len(argu) < 1:
                           result = 'Please specify projectname'
                           status = 'ERROR'
                       else:
                           projectname =  argu[0].childNodes[0].data
                           p = ProjectManager.GetProject(projectname,self.c)
                           if p != 'not register to the project yet' and p != 'DB not open yet': 
                               response = DBcommand.do_defCommand(p.fileobj,command,argu,id)

                           else:
                               result = p
                               status = 'ERROR'
                               response = response_wrapper(status,result,id)
                   
                       try:
                           self._sock.sendall(response)
                           messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response) 
                       except exceptions.Exception,x:
                           messager.output(str(x))
                           ProjectManager.DelClient(self.c)
                           break
                       
                   elif command in self._sql_newrecord_command:
                       if self._sqldb != None:
                           return_result = DBcommand. do_sql_newrecord_command(self._sqldb,command,argu)
        
                           result = return_result[0]
                           status = return_result[1]
                           table = return_result[2]
                           recordid = return_result[3]
                          
                       else:
                           status = 'ERROR'
                           result = 'DB not open yet'

                       response = response_wrapper(status,result,id)
                           
                       try:
                           self._sock.sendall(response)
                           messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                           if status == 'OK':
                               message = '<db_broadcast><message>New Record in table \'' + table+'\'. record id: '+recordid + '</message></db_broadcast>\n'
                               messager.output('# Handler message : broadcast '+message )
                               clientlist = ProjectManager.GetClientlist()
                               # check project associated clients, send broadcast message
                               for client in clientlist:
                                   if client.GetDb() ==  self.c.GetDb():
                                       if client.broadcast == True:
                                           client.sock.sendall(message)
                                                                                                                                 
                       except exceptions.Exception,x:
                           messager.output(str(x))
                           ProjectManager.DelClient(self.c)
                           break

                   elif command in self._sql_setdata_command:
                       if self._sqldb != None:
                           return_result = DBcommand. do_sql_setdata_command(self._sqldb,command,argu)
                           result = return_result[0]
                           status = return_result[1]
                           table = return_result[2]
                           recordid = return_result[3]
                          
                       else:
                           status = 'ERROR'
                           result = 'DB not open yet'
                       response = response_wrapper(status,result,id)
                           
                       try:
                           self._sock.sendall(response)
                           messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)
                           
                           if status == 'OK':
                               message = '<db_broadcast><message>Update table \'' + table+'\'. Set data in record id: '+recordid + '</message></db_broadcast>\n'
                               messager.output('# Handler message : broadcast '+message )
                               clientlist = ProjectManager.GetClientlist()
                               # check project associated clients, send broadcast message
                               for client in clientlist:
                                   if client.GetDb() ==  self.c.GetDb():
                                       if client.broadcast == True:
                                           client.sock.sendall(message)
                                                                                                                                 
                       except exceptions.Exception,x:
                           messager.output(str(x))
                           ProjectManager.DelClient(self.c)
                           break

                   else:
                       if self._sqldb != None:
                           response = DBcommand. do_sql_command(self._sqldb,command,argu,id)
                          
                       else:
                           status = 'ERROR'
                           result = 'DB not open yet'
                           response = response_wrapper(status,result,id)
                           
                       try:
                           self._sock.sendall(response)
                           messager.output('# Handler message : in connection '+ str(self._addr) + ' sent response: '+ response)     
                       except exceptions.Exception,x:
                           messager.output(str(x))
                           ProjectManager.DelClient(self.c)
                           break
                       
           except socket.timeout:
               # timeout, check event flag
               if en.isSet():
                   # asked to exit, warning client
                   messager.output('# Handler message: child thread is told shutting down')
                   break
               
               else:
                  continue
               
           except exceptions.Exception,x:
                messager.output(str(x))
                ProjectManager.DelClient(self.c)
                break

        
        ## close connection and thread exit
        if self._sqldb != None:
            self._sqldb.CloseDB()
       
        self._sock.close()
        messager.output('#Handler message :'+  str(self._addr) + ' disconnected. ')


def sockfactory(lockfile,shutdown):
    ''' generate socket & create a lockfile '''
    
    #### check whether the lockfile exists
    if os.path.isfile(lockfile):
        messager.output('# Handler message: find lockfile')
        f = open(lockfile,"r")
        a = f.readline()
        port = a[15:]
        messager.output('# Handler message: port number is: '+ str(port))

        ## try to bind the port number, if failed, exit. or replace lockfile.
        socket.setdefaulttimeout(5)
        s = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
        s.setsockopt(socket.SOL_SOCKET,socket.SO_REUSEADDR,1)
        try:
            s.bind(('',int(port)))
        except exceptions.Exception,e:
            messager.output(str(e))
            messager.output('# Handler message: there is one handler running')
            print 'There is another handler running on port '+port+'.'
            sys.exit(0)
    else:
        ## creat a socket
        socket.setdefaulttimeout(30)
        s = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
        s.setsockopt(socket.SOL_SOCKET,socket.SO_REUSEADDR,1)

        port = 4090
        host = ''
        while 1:
            try:
                s.bind(('',port))
                messager.output('# Handler message: port number is '+ str(port))
                break
            except exceptions.Exception,e:
                messager.output(' # Handler message: trying to use different port number')
                messager.output(str(e))
                port = port + 1
                
    ## write to a lockfile
    try:      
        f = open(lockfile, "w")
        f.write("port number is:")
        f.write(str(port))
        f.close()
    except exceptions.Exception,e:
        print str(e)
        
    s.listen(5)
    return s

if __name__ == '__main__':

    shutdown = False
    if '-shutdown' in sys.argv:
        shutdown = True
        
    if '-debug' in sys.argv:
        messager = messager(True)
    else:
        messager = messager(False)
    # create a lock file
    lockfile = os.path.join(ccp4i.GetDotCCP4(),"dbccp4i.LOCK")
    # create a socket
    s = sockfactory(lockfile,shutdown)
    # create an event object to communicate between threads
    en = threading.Event()
    
    try:
      print '================================================================'  
      print 'DB Handler started at', datetime.datetime.now()
      print 'Version: ', __version__
      print '================================================================'
      messager.output('====================================================================')
      messager.output('# Handler message: DB Handler started at '+str(datetime.datetime.now() ))
      messager.output('Version: '+ __version__)
      messager.output('====================================================================')
      while 1:      
          try:
              clientsock,clientaddr = s.accept()
              messager.output('# Handler message: New connect from '+ str(clientaddr))
              
              t = MyThread(clientsock,clientaddr)
              t.setDaemon(0)
              t.start()
           
          except socket.timeout:
              if ProjectManager.NumClient() == 0:
                  break

              # still have clients, check any message from child threads,
              # if yes,exit; otherwise continue
              else:
                  #messager.output('# Handler message: Client connected: '+ str(ProjectManager.GetClientlist()))
                  if en.isSet():
                      break
                  else:
                     try:
                         ## save to disk
                         ProjectManager.SaveProject()
                         ProjectManager.SaveDirectory()
                       
                     except exceptions.Exception,e:
                         messager.output(str(e))
                     continue
          
          except KeyboardInterrupt:
               messager.output('Caught a keyboard interrupt')
               break

          except exceptions.Exception,e:
              messager.output('caught exception')
              break
          
    finally:
      # do cleaning: close all projects, close socket
      ProjectManager.Clean()
      s.close()
      os.remove(lockfile)
    print '======================================================================'
    print 'DB Handler stopped at',  str(datetime.datetime.now())
    print '======================================================================'
    messager.output('=================================================================')
    messager.output('# Handler message: Handler stopped at '+ str(datetime.datetime.now()))
    messager.output('=================================================================\n')

