#
#     Copyright (C) 2006
#
#     This code is distributed under the terms and conditions of the
#     CCP4 licence agreement as `Part 1' (Annex 2) software.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#====================================================================
# manager.py
#
# Module for handling client & project management in handler
#
# Wanjuan Yang March 06
#
#====================================================================

#CCP4i_cvs_Id $Id: manager.py,v 1.12 2007/01/18 11:49:54 wy45 Exp $

__version__ = "$Revision: 1.12 $"


#######################################################################
# Import modules that this module depends on
#######################################################################
import ccp4i
import xml.sax.saxutils as su
import os
#######################################################################
# Class definitions
#######################################################################

class Manager:
    '''  A class for handlering project & client management in handler.'''
    def __init__(self):
        self._projectlist = []
        self._clientlist = []
        self._userlist = []
        # create a directory object but not load the data
        self.directory = ccp4i.directories()
        self.dataloaded = False
        
    def NumProject(self):
        return len(self._projectlist)

    def NumClient(self):
        return len(self._clientlist)

    def NumUser(self):
        return len(self._userlist)

    def SaveProject(self):
        if self.NumProject() >0:
            for p in self._projectlist:
                p.save()
                
    def SaveDirectory(self):
        if self.dataloaded:
            self.directory.save()
        
    def OpenProject(self,projectname,client,grablock):
        if  self.IsProjectOpened(projectname) != True:
          if self.dataloaded:
            dir = self.directory.projectdir(projectname)
            db = ccp4i.database(projectname,dir)
            if db.exists(): #check if exist
                if db.islocked() == False or grablock == True:
                    result = db.open(grablock)
                    project = Project(projectname,db)
                    ## add client to the project 
                    project.AddClient(client)
                    self._projectlist.append(project)
                else:       
                    result = 'data is locked'
            else:
                result = 'DB not exist'
          else:
             result = 'Data not loaded'
             
          if result == False:
              result =  db.getmessage()
          return result
               
            
        else:
           # attach to client to the project. if client not in projectlist
           for p in self._projectlist:
               if p.GetProjectname() == projectname:
                   p.AddClient(client)
                   return True

           else:
                return False

    def NewProject(self,projectname,dir,client):
        if  self.IsProjectOpened(projectname) != True:
            db = ccp4i.database(projectname,dir)
            if db.exists() != True:
                result = db.create()
                if self.dataloaded:
                    self.directory.addprojectref(projectname,dir)
                project = Project(projectname,db)
                ## add client to the project 
                project.AddClient(client)
                self._projectlist.append(project)
            else:
                result = 'DB already exists'
            
        else:
            result = 'DB already exists and opened'

        return result
        
    def GetProject(self,projectname,client):
        if self.IsProjectOpened(projectname):
            for p in self._projectlist:
                if p.projectname == projectname:
                    if client in p.clientlist:
                        return p
                    else:
                        return 'not register to the project yet'
            else:
                return 'DB not open yet'
        else:
            return 'DB not open yet'

    def GetProjectlist(self):
        return self._projectlist

    def IsProjectOpened(self,projectname):
        for p in self._projectlist:
            if p.projectname == projectname:
                return True
        else:
            return False

    def AddClient(self,client):
        self._clientlist.append(client)

    def GetClientlist(self):
        return self._clientlist
    
    def broadcast(self,message,projectname):
        for p in self._projectlist:
            if p.projectname == projectname:
                for client in p.clientlist:
                    if client.GetBroadcast() == True:
                        client.sock.send(message)
    
    def CreateUser(self,username,pid,client):
        user = User(username,pid)
        self._userlist.append(user)
        user.AddClient(client)

    def IsUserRegistered(self,username):
        for user in self._userlist:
            if user.GetUsername() == username:
                return True
        else:
            return False
        
    def AttachClientToUser(self,username, client):
        for user in self._userlist:
            if user.GetUsername() == username:
                user.AddClient(client)
                
    def DelClient(self,client):
        ## del the client in the list
        if client in self._clientlist:     
            no = self._clientlist.index(client)
            del self._clientlist[no]
         
            
        ## del the client from the projects it has associated 
        for p in self._projectlist:
            if client in p.clientlist:
                p.DeleClient(client)
                # if the client deleted is the last one user of the project, close the project
                if len(p.clientlist) == 0:
                    del self._projectlist[self._projectlist.index(p)]
                    p.close()
        

    def CloseProject(self,projectname,client):
        for p in self._projectlist:
            if p.projectname == projectname:
                p.DeleClient(client)
                if len(p.clientlist) == 0:
                    n = self._projectlist.index(p)
                    del self._projectlist[n]
                    p.close()
        
    def Clean(self):
        for p in self._projectlist:
            p.close()

        if self.directory.islocked():
            self.directory.release()


class Project:
    '''A class which has a database instance and the clients associated with the database instance'''
    
    def __init__(self,projectname,fileobj):
        self.projectname = projectname
        self.backend = None
        self.clientlist = []
        self.fileobj = fileobj

    def __repr__(self):
        return self.projectname
    
    def SetBackend(self,backend):
        self.backend = backend

    def getbackend(self):
        return self.backend   

    def GetProjectname(self):
        return self.projectname
    
    def AddClient(self,client):
        self.clientlist.append(client)

    def DeleClient(self,client):
        i = self.clientlist.index(client)
        del self.clientlist[i]

    def save(self):
        self.fileobj.save()
        
    def close(self):
        self.fileobj.close()


class Client:
    ''' A class which represent a client connection and have associated attributes, e.g. what projects the client is opening, what application is the client using, is it require broadcast or not etc.'''
    
    def __init__(self,sock,addr):
        self.sock = sock
        self.addr = addr
        self._auth = None
        self._application = None
        self._projectlist = []
        self.broadcast = True
        self._user = None
        self._db = None

    def __repr__(self):
        return str(self.addr)
        
    def AddProject(self,projectname):
        self._projectlist.append(projectname)

    def GetProjects(self):
        return self._projectlist

    def DeleProject(self,projectname):
        n = self._projectlist.index(projectname)
        del self._projectlist[n]

    def AddUser(self,username):
        self._user = username

    def GetUser(self):
        return self._user
    
    def IsProjectOpened(self,projectname):
        if projectname in self._projectlist:
            return True
        else:
            return False
    
    def SetAuth(self,auth):
        self._auth = auth

    def GetAuth(self):
        return self._auth

    def SetApplication(self,application):
        self._application = application

    def GetApplication(self):
        return self._application

    def SetDb(self,dbname):
        self._db = dbname

    def GetDb(self):
        return self._db

    def SetBroadcast(self,flag):
        self.broadcast = flag

    def GetBroadcast(self):
        return self.broadcast
  
class User:
    '''  A class that represent a user. '''
    def __init__(self,username,pid):
        self._user = username
        self._pid = pid
        self.clientlist = []
   
    def __repr__(self):
        return self._user

    def GetUsername(self):
        return self._user
    
    def AddClient(self,client):
        self.clientlist.append(client)

    def GetClients(self):
        return self.clientlist

    def DeleClient(self,client):
        n = self.clientlist.index(client)
        del self.clientlist[n]


class messager:
    ''' A class that outputs the message to a file or on the screen '''

    def __init__(self,flag=False):
        self.flag = flag
        self.file = open(os.path.join(ccp4i.GetDotCCP4(),'dbhandler.log'),'a')
    def output(self,message):
        if self.flag:
            print message
        else:
            self.file.write(message)
            self.file.write(' \n')
            
    def close(self):
        self.file.close()


def response_wrapper(status,result,id):
	''' wrap the message '''
	if type(result) == tuple or type(result) == list:
		response = "<db_response><status>"+ status + "</status><result>"+ result_wrapper(result)+"</result><response_id>"+ str(id) + "</response_id></db_response>\n"

	else:
		response = "<db_response><status>"+status+"</status><result>"+ str(result) + "</result><response_id>" + str(id) + "</response_id></db_response>\n"
	return response	

def result_wrapper(result):
	''' wrap the result '''
        tmp = "<list>"
        
	for item in result:
		if type(item) == tuple or type(item) == list:
			tmp = tmp + "<item>"+result_wrapper(item)+"</item>"
		else:
			tmp = tmp + "<item>"+su.escape(str(item))+"</item>"

	tmp = tmp + "</list>"
	return tmp


ProjectManager = Manager()

